/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */

/* 
 * Copyright (C) 2002 James Willcox
 * Copyright (C) 2003 Ximian, Inc
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more av.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

#include <config.h>

#include <string.h>
#include <bonobo-activation/bonobo-activation.h>
#include <gtk/gtk.h>
#include <glade/glade-xml.h>
#include <libgnomevfs/gnome-vfs-utils.h>
#include <libbonobo.h>

#include <stdlib.h>

#define VIEW_IID_MAILTO      "OAFIID:Nautilus_Mailto"

#define NAUTILUS_TYPE_MAILTO	     (nautilus_mailto_get_type ())

#define NAUTILUS_MAILTO(obj)	     (GTK_CHECK_CAST ((obj), NAUTILUS_TYPE_MAILTO, NautilusMailto))

typedef struct {
	BonoboObject parent;
} NautilusMailto;

typedef struct {
	BonoboObjectClass parent;

	POA_Bonobo_Listener__epv epv;
} NautilusMailtoClass;

static GType nautilus_mailto_get_type (void);

static char *
get_path_from_url (const char *url)
{
	GnomeVFSURI *uri     = NULL;
	char        *path    = NULL;
	
	uri = gnome_vfs_uri_new (url);

	if (uri != NULL) {
		path = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
	}

	if (uri != NULL)
		gnome_vfs_uri_unref (uri);
	
	return path;
}

static void
impl_Bonobo_Listener_event (PortableServer_Servant servant,
			    const CORBA_char *event_name,
			    const CORBA_any *args,
			    CORBA_Environment *ev)
{
	NautilusMailto *mailto;
	const CORBA_sequence_CORBA_string *list;
	guint i;
	GString *argument;
	char *path;
	char *evo_args[3];
	GtkWidget *dialog;
	GError *error = NULL;
	
	mailto = NAUTILUS_MAILTO (bonobo_object_from_servant (servant));
	
	if (!CORBA_TypeCode_equivalent (args->_type, TC_CORBA_sequence_CORBA_string, ev)) {
		return;
	}

	list = (CORBA_sequence_CORBA_string *)args->_value;

	argument = g_string_new ("mailto:?");

	for (i = 0; i < list->_length; i++) {
		if (i > 0) {
			g_string_append_c (argument, '&');
		}
		path = get_path_from_url (list->_buffer[i]);
		g_string_append_printf (argument, "attach=%s", path);
		g_free (path);
	}
	
	evo_args[0] = "evolution-1.4";
	evo_args[1] = argument->str;
	evo_args[2] = NULL;
	g_spawn_async (NULL,
		       evo_args, 
		       NULL,
		       G_SPAWN_SEARCH_PATH,
		       NULL,
		       NULL,
		       NULL,
		       &error);

	if (error) {
		dialog = gtk_message_dialog_new (NULL,
						 0,
						 GTK_MESSAGE_ERROR,
						 GTK_BUTTONS_OK,
						 _("Couldn't start Evolution: %s"), error->message);
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog); 

		g_error_free (error);
	}
		       
	g_string_free (argument, TRUE);
}


/* initialize the class */
static void
nautilus_mailto_class_init (NautilusMailtoClass *class)
{
	POA_Bonobo_Listener__epv *epv = &class->epv;
	epv->event = impl_Bonobo_Listener_event;
}


static void
nautilus_mailto_init (NautilusMailto *mailto)
{
}


BONOBO_TYPE_FUNC_FULL (NautilusMailto, 
		       Bonobo_Listener, 
		       BONOBO_TYPE_OBJECT,
		       nautilus_mailto);


static CORBA_Object
mailto_shlib_make_object (PortableServer_POA poa,
			  const char *iid,
			  gpointer impl_ptr,
			  CORBA_Environment *ev)
{
	NautilusMailto *mailto;

	mailto = g_object_new (NAUTILUS_TYPE_MAILTO, NULL);

	bonobo_activation_plugin_use (poa, impl_ptr);

	return CORBA_Object_duplicate (BONOBO_OBJREF (mailto), ev);
}

static const BonoboActivationPluginObject plugin_list[] = {
	{ VIEW_IID_MAILTO, mailto_shlib_make_object },
	{ NULL }
};

const BonoboActivationPlugin Bonobo_Plugin_info = {
	plugin_list,
	"Nautilus Mailto Component"
};
